@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp,v 1.6 2006/02/13 17:39:31 rblazek Exp $
+ * $Id: grass57dataset.cpp 11554 2007-05-18 17:09:03Z mloskot $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -28,29 +28,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: grass57dataset.cpp,v $
- * Revision 1.6  2006/02/13 17:39:31  rblazek
- * header files moved to grass
- *
- * Revision 1.5  2005/09/30 21:07:57  fwarmerdam
- * applied patches from bug 822, add georef support
- *
- * Revision 1.4  2005/05/06 18:21:14  fwarmerdam
- * Applied patch from Radim.  Better logic to reset the region.
- *
- * Revision 1.3  2004/09/24 14:28:49  fwarmerdam
- * fixed some typos with floating point (from Radim)
- *
- * Revision 1.2  2004/09/22 20:39:15  fwarmerdam
- * some updates to reduce spurious error reports
- *
- * Revision 1.1  2004/09/13 18:18:26  fwarmerdam
- * Variant code from Radim that works with GRASS 5.7 libraries directly.
- * Currently coordinate system support is broken with this version.
- *
- */
+ ****************************************************************************/
 
 #include <stdlib.h>
 
@@ -63,6 +41,7 @@ extern "C" {
 #undef class
 #endif
     
+#include <grass/version.h>
 #include <grass/gprojects.h>
 #include <grass/gis.h>
 }
@@ -73,7 +52,7 @@ extern "C" {
 
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp,v 1.6 2006/02/13 17:39:31 rblazek Exp $");
+CPL_CVSID("$Id: grass57dataset.cpp 11554 2007-05-18 17:09:03Z mloskot $");
 
 CPL_C_START
 void	GDALRegister_GRASS(void);
@@ -409,7 +388,10 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	 sNewWindow->ew_res != sOpenWindow.ew_res || sNewWindow->ns_res != sOpenWindow.ns_res ||
 	 sNewWindow->rows   != sOpenWindow.rows   || sNewWindow->cols   != sOpenWindow.cols )
     {
-        G_close_cell( hCell );
+	if( hCell >= 0 ) {
+            G_close_cell( hCell );
+	    hCell = -1;
+	}
 
 	/* Set window */
 	G_set_window( sNewWindow );
@@ -423,6 +405,7 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	
 	if ( (hCell = G_open_cell_old( pszCellName, pszMapset)) < 0 ) {
 	    CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster '%s'", pszCellName );
+            this->valid = false;
 	    return CE_Failure;
 	}
 
@@ -459,6 +442,8 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImage )
 
 {
+    if ( ! this->valid ) return CE_Failure;
+
     // Reset window because IRasterIO could be previosly called
     if ( ResetReading ( &(((GRASSDataset *)poDS)->sCellInfo) ) != CE_None ) {
        return CE_Failure;
@@ -510,6 +495,8 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
     /* Calculate the region */
     struct Cell_head sWindow;
     struct Cell_head *psDsWindow;
+    
+    if ( ! this->valid ) return CE_Failure;
 
     psDsWindow = &(((GRASSDataset *)poDS)->sCellInfo);
     
@@ -526,7 +513,10 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
     /* Reset resolution */
     G_adjust_Cell_head ( &sWindow, 1, 1);
 
-    ResetReading ( &sWindow );
+    if ( ResetReading ( &sWindow ) != CE_None )
+    {
+        return CE_Failure;
+    }
     
     /* Read Data */
     CELL  *cbuf = NULL;
@@ -796,7 +786,11 @@ bool GRASSDataset::SplitPath( char *path, char **gisdbase, char **location,
 /*                                Open()                                */
 /************************************************************************/
 
+#if GRASS_VERSION_MAJOR  >= 6 && GRASS_VERSION_MINOR  >= 3
+typedef int (*GrassErrorHandler)(const char *, int);
+#else
 typedef int (*GrassErrorHandler)(char *, int);
+#endif
 
 GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
